import { LitElement, css, html } from 'lit';
import { customElement, property } from 'lit/decorators.js';

import '@components/EveLink';
import { when } from 'lit/directives/when.js';

@customElement('arx-app-icon')
export class AppIcon extends LitElement {
  @property()
  icon: string | undefined;

  @property()
  color = '#ff9900';

  @property()
  href = '#';

  @property()
  name = 'App';

  private iconElement?: HTMLElement;

  static override styles = [
    css`
      :host {
        display: flex;
        --animation-speed: 0.25s;
      }

      .app-icon {
        display: flex;
        flex-direction: column;
        align-items: center;
        gap: 8px;
        text-decoration: none;
        padding: 6px;
        border-radius: 18px;
        transition: transform var(--animation-speed)
          cubic-bezier(0.175, 0.885, 0.32, 1.275);
      }

      .app-icon:hover {
        transform: translateY(-2px);
      }

      .icon {
        width: 96px;
        height: 96px;
        border-radius: 24px;
        box-shadow: 0 3px 8px rgba(0, 0, 0, 0.12);
        display: flex;
        justify-content: center;
        align-items: center;
        border: 1px solid rgba(255, 255, 255, 0.2);
        overflow: hidden;
        position: relative;
        transition: all var(--animation-speed) ease-out;
        &::before {
          content: "";
          position: absolute;
          inset: 0;
          background: linear-gradient(
            145deg,
            rgba(255, 255, 255, 0.25) 20%,
            rgba(255, 255, 255, 0) 60%
          );
          pointer-events: none;
          opacity: 1;
          transition: opacity var(--animation-speed) ease-out;
        }

        &::after {
          content: "";
          position: absolute;
          inset: 0;
          background: linear-gradient(
            var(--gradient-angle),
            rgba(255, 255, 255, 0.25) 20%,
            rgba(255, 255, 255, 0) 60%
          );
          pointer-events: none;
          opacity: 0;
          transition: opacity var(--animation-speed) ease-out;
        }

        &:hover::before {
          opacity: 0;
        }

        &:hover::after {
          opacity: 1;
        }
      }

      .icon-svg {
        color: white;
        filter: drop-shadow(
          calc(cos(var(--gradient-angle)) * 2px)
            calc(sin(var(--gradient-angle)) * 2px) rgba(0, 0, 0, 0.2)
        );
        display: flex;
        justify-content: center;
        align-items: center;
        width: 100%;
        height: 100%;
      }

      .app-name {
        display: block;
        width: 100%;
        text-align: center;
        font-size: 18px;
        font-weight: 500;
        color: var(--color-base-content);
        white-space: nowrap;
        text-decoration: none;
        transition: all var(--animation-speed) ease-out;
      }

      .app-icon:hover .icon {
        transform: scale(1.08);
        box-shadow: 0 6px 12px rgba(0, 0, 0, 0.25);
      }

      .app-icon:active .icon {
        transform: scale(0.96);
      }
    `,
  ];

  override firstUpdated() {
    this.iconElement = this.shadowRoot?.querySelector('.icon') as HTMLElement;
    if (this.iconElement) {
      this.iconElement.addEventListener('mousemove', this.handleMouseMove);
    }
  }

  override disconnectedCallback() {
    super.disconnectedCallback();
    if (this.iconElement) {
      this.iconElement.removeEventListener('mousemove', this.handleMouseMove);
    }
  }

  handleMouseMove = (e: MouseEvent) => {
    if (!this.iconElement) return;

    const rect = this.iconElement.getBoundingClientRect();
    const centerX = rect.left + rect.width / 2;
    const centerY = rect.top + rect.height / 2;

    const dx = e.clientX - centerX;
    const dy = e.clientY - centerY;
    const angle = Math.atan2(dy, dx) * (180 / Math.PI);

    this.iconElement.style.setProperty('--gradient-angle', `${angle}deg`);
  };

  override render() {
    return html`
      <arx-eve-link href="${this.href}" class="app-icon">
        <div class="icon" style="background: ${this.color};">
          ${when(
            this.icon,
            () => html`<iconify-icon
              icon="${this.icon}"
              class="icon-svg"
              width="64"
              height="64"
            ></iconify-icon>`,
          )}
        </div>
        <span class="app-name">${this.name}</span>
      </arx-eve-link>
    `;
  }
}
